/* A more complicated example for the SOR algorithm: the equation of motion 
   of field theories  in cylindrical coordinates 

v1.0: Poisson's equation in cylindrical coordinates, (rho,z). We work in the 
quarter plane rho>0, z>=0 and place a uniformly charged ring on the x-y plane.
The ring has charge density unity and radius R and acts as a source
 
I update all points. The symmetry of the problem immediately implies that 
the potential is symmetric w.r.t. reflections z<-> -z, since the electric field should be antisymmetric!

I've found that omega=1.3 leads to an error of 10^(-18) after ca. 12000 steps

v1.0: I now include a Higgs potential. This means that the scalar field becomes massive. The boundary conditions at infinity also change. There the field should take its vev.

The parameter values shown here indeed work!

This is valid for the neutral scalar field. 

v2.0 I display the potential energy density. The vev is normalized to 1.0 and the density is normalized to 1.0 at the origin. 
*/


#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "test_param_neutral_scalar.h"

double f_new[NRHO+1][NZ+1], f_old[NRHO+1][NZ+1];


void init(){
    int i,j;
    
    for(i=1;i<NRHO;i++){
	for(j=1;j<NZ;j++){
	    f_old[i][j] = 0.0;
	    f_new[i][j] = 0.0;
	}
    }
}

void bc(){
    int i, j;

// I impose that the zero of the Higgs field is at the origin:

    f_new[0][0] = 0.0;
    f_old[0][0] = 0.0;


/* I impose that the Higgs field attain its vev at infinity. 
   there are two boundaries: j=NZ at i fixed and i=NRHO at j fixed

    for(i=1;i<=NRHO;i++){
	f_new[i][NZ] = 1.0;
	f_old[i][NZ] = 1.0;
    }

    for(j=1;j<=NZ;j++){
	f_new[NRHO][j] = 1.0;
	f_old[NRHO][j] = 1.0;
    }
*/

    for(i=1;i<=NRHO;i++){
	f_new[i][NZ] = 0.0;
	f_old[i][NZ] = 0.0;
    }

    for(j=1;j<=NZ;j++){
	f_new[NRHO][j] = 0.0;
	f_old[NRHO][j] = 0.0;
    }

	
}

double Vder(int i, int j){
    return(beta*f_new[i][j]*( sqr(f_new[i][j])-1.0 ));
}


double J(int i, int j){// the source
    return(  ( ((i==NRJ) && (j==NZJ)) ? RHO : 0.0) );
}


void update(double omega){
    int i, j;

    double crz;
    double iplus, iminus;
    double f1,f2,f3,f4;

// i=0, j>0
    i=0;
    iplus = 0.5; 
    crz = (rho(iplus))*inv_ar2 + 2.0*rho(iplus)*inv_az2;
    crz = 1.0/crz;
    for(j=1;j<NZ;j++){
	f1 = (1.-omega)*f_new[0][j];
	f2 = inv_ar2*( rho(iplus)*f_new[1][j]);
	f3 = rho(iplus)*inv_ar2*(f_new[0][j+1]+f_new[0][j-1]);
	f4 = rho(iplus)*(J(0,j)+Vder(0,j));

	f_new[0][j] = f1 + omega*crz*(f2+f3+f4);
	}

   
	
// i>0

    for(i=1;i<NRHO;i++){
	iplus = (double)i+0.5;
	iminus = (double)i-0.5;
	crz = (rho(iplus)+rho(iminus))*inv_ar2 + 2.0*rho(iplus)*inv_az2;
	crz = 1.0/crz;

// j=0

	f1 = (1.-omega)*f_new[i][0];
	f2 = inv_ar2*( rho(iplus)*f_new[i+1][0] + rho(iminus)*f_new[i-1][0]);
	f3 = rho(iplus)*inv_ar2*(f_new[i][1]);
	f4 = rho(iplus)*(J(i,0)+Vder(i,0));

	f_new[i][0] = f1 + omega*crz*(f2+f3+f4);

// j>0

	for(j=1;j<NZ;j++){
	    
	f1 = (1.-omega)*f_new[i][j];
	f2 = inv_ar2*( rho(iplus)*f_new[i+1][j] + rho(iminus)*f_new[i-1][j]);
	f3 = rho(iplus)*inv_ar2*(f_new[i][j+1]+f_new[i][j-1]);
	f4 = rho(iplus)*(J(i,j)+Vder(i,j));

	f_new[i][j] = f1 + omega*crz*(f2+f3+f4);
	}
    }
}

void replace(){
    int i,j;
    for(i=0;i<NRHO;i++){
	for(j=0;j<NZ;j++){
	    f_old[i][j] = f_new[i][j];
	}
    }

}

double error(){
    int i, j;
    double temp = 0.0; 

    for(i=0;i<NRHO;i++){
	for(j=0;j<NZ;j++){
	    temp = temp + sqr(f_new[i][j]-f_old[i][j]);
	}
    }

    return(temp/SIZE);
}



void output(){
    FILE *SOL;
    SOL = fopen("solscalar_TEST.out","w");
    int i, j;
    for(i=0;i<NRHO;i++){
	for(j=0;j<NZ;j++){
	    fprintf(SOL,"%g\t%g\t%g\n",i*a_rho, j*a_z, f_new[i][j]);
	}
    }

    fclose(SOL);
}



void energy_density(){
    FILE *EDEN;
      EDEN = fopen("edensity_pot_scalar_TEST.out", "w");
    double edensity_cin_rho, edensity_cin_z,edensity_pot;
    double edensity;
    double iplus, iminus;
    int i,j;
    

    double sum = 0.0; 
    double virial = 0.0;
    for(i=1;i<NRHO;i++){
	iplus = (double)i + 0.5;
	iminus = (double)i - 0.5;
	for(j=1;j<NZ;j++){
	    edensity_pot = -rho(iplus)*0.25*beta*sqr( sqr(f_new[i][j])-1.0 );
	    edensity_cin_z = rho(iplus)*0.5*sqr( (f_new[i][j+1]-f_new[i][j])/a_z);


	    edensity_cin_rho = rho(iplus)*0.5*sqr( (f_new[i+1][j]-f_new[i][j])/a_rho);


	    edensity = a_rho*a_z*(-edensity_pot + edensity_cin_rho + edensity_cin_z);
	    virial = virial + edensity;
/*	    printf("%g\t%g\t%g\n",i*a_rho, j*a_z, edensity);
	    printf("%g\t%g\t%g\n",i*a_rho, j*a_z, a_rho*a_z*edensity_pot); */
	    fprintf(EDEN,"%g\t%g\t%g\n",i*a_rho, j*a_z,sqr(sqr(f_new[i][j])-1.0));
	    sum = sum + a_rho*a_z*( rho(iplus)*sqr( sqr(f_new[i][j])-1.0 ) );
	}
    }
//    printf("total (potential) energy = %g\n", sum);
    printf("%g\t%g\t%g\n", -beta, sum, virial);
    fclose(EDEN);
}

main(){

    FILE *ERR;
      ERR = fopen("error_scalar_TEST.out", "w");

    int k;

    double omega;
    scanf("%lf",&omega);


//    for(beta=0.0;beta> -20.0;beta = beta - 1.0){

    init(); //initial guess
    bc(); //boundary conditions





    
    double final_error;

    for(k=1;k<M;k++){
	update(omega);
	
	fprintf(ERR,"%d\t%g\n",k,sqrt(error()));

	final_error = sqrt(error());

	replace();
	
    }

    fclose(ERR);

    output();

    printf("%g\t",final_error);
    energy_density();

//    }
}
